import typing

from aad_fastapi.roles.all_role_validator import AllRoleValidator
from aad_fastapi.roles.any_role_validator import AnyRoleValidator
from aad_fastapi.roles.role_requirement import RoleRequirement


class RoleValidator:
    """Role validator class"""

    _validators = {
        RoleRequirement.ALL: AllRoleValidator,
        RoleRequirement.ANY: AnyRoleValidator,
    }

    def __init__(
        self, mandatory_roles: typing.List[str], role_requirement: RoleRequirement
    ):
        self.mandatory_roles = mandatory_roles
        self.role_requirement = role_requirement
        self.validator_class = self._validators.get(role_requirement)
        if self.validator_class is None:
            raise ValueError(f"Invalid role requirement: {role_requirement}")

    def validate_roles(self, user_roles: typing.List[str]) -> bool:
        """validate the user roles against the mandatory roles"""
        validator = self.validator_class()
        return validator.validate_roles(user_roles, self.mandatory_roles)



import pytest

from aad_fastapi.roles.all_role_validator import AllRoleValidator
from aad_fastapi.roles.any_role_validator import AnyRoleValidator
from aad_fastapi.roles.role_requirement import RoleRequirement
from aad_fastapi.roles.role_validator import RoleValidator


@pytest.mark.parametrize(
    "roles, requirement, user_roles, expected",
    [
        (["admin", "editor"], RoleRequirement.ALL, ["admin", "editor"], True),
        (["admin", "editor"], RoleRequirement.ALL, ["admin"], False),
        (["admin", "editor"], RoleRequirement.ANY, ["admin"], True),
        (["admin", "editor"], RoleRequirement.ANY, ["guest"], False),
    ],
)
def test_role_validator(roles, requirement, user_roles, expected):
    validator = RoleValidator(roles, requirement)
    assert validator.validate_roles(user_roles) == expected


def test_role_validator_invalid_role_requirement():
    with pytest.raises(ValueError):
        RoleValidator([], "invalid")


@pytest.mark.parametrize(
    "mandatory_roles, user_roles, expected",
    [
        (["admin", "editor"], ["admin", "editor"], True),
        (["admin", "editor"], ["admin"], False),
    ],
)
def test_all_role_validator(mandatory_roles, user_roles, expected):
    validator = AllRoleValidator()
    assert validator.validate_roles(user_roles, mandatory_roles) == expected


@pytest.mark.parametrize(
    "mandatory_roles, user_roles, expected",
    [
        (["admin", "editor"], ["admin"], True),
        (["admin", "editor"], ["guest"], False),
    ],
)
def test_any_role_validator(mandatory_roles, user_roles, expected):
    validator = AnyRoleValidator()
    assert validator.validate_roles(user_roles, mandatory_roles) == expected



from aad_fastapi.aad_vault import AadVault


def test_get_msal_client_credential_when_x_type_is_x_pem_file():
    vault = AadVault("https://test.vault.azure.net")

    from cryptography.hazmat.primitives.asymmetric import rsa
    from cryptography.hazmat.backends import default_backend
    from cryptography.hazmat.primitives import serialization

    rsa_private_key = rsa.generate_private_key(
        public_exponent=65537, key_size=2048, backend=default_backend()
    )
    private_key = rsa_private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption(),
    ).decode("utf-8")

    mock_certificate = {
        "thumbprint": "test_thumbprint",
        "private_key": private_key,
        "x_type": "x-pem-file",
    }

    client_credentials = vault.get_msal_client_credential(mock_certificate)

    assert client_credentials["thumbprint"] == "test_thumbprint"
    key = client_credentials["private_key"]
    assert isinstance(key, rsa.RSAPrivateKey)
    assert key.key_size == 2048
    assert key.public_key().key_size == 2048
    assert key.public_key().public_numbers().e == 65537



import typing

from aad_fastapi.roles.role_validator_interface import RoleValidatorInterface


class AnyRoleValidator(RoleValidatorInterface):
    """Validate that at least one mandatory role is present in the user roles"""

    def validate_roles(
        self, user_roles: typing.Sequence[str], mandatory_roles: typing.Sequence[str]
    ) -> bool:
        return any(mandatory_role in user_roles for mandatory_role in mandatory_roles)



import inspect
import typing
from functools import wraps
from typing import Optional

from fastapi.openapi.models import OAuthFlowAuthorizationCode, OAuthFlows
from fastapi.security import OAuth2
from starlette.authentication import requires
from starlette.exceptions import HTTPException
from starlette.requests import Request

from .aad_options import AzureAdSettings
from .aad_user import AadUser
from .roles.role_requirement import RoleRequirement
from .roles.role_validator import RoleValidator


def authorize(
    scopes: typing.Union[str, typing.Sequence[str]] = None,
    roles: typing.Union[str, typing.Sequence[str]] = None,
    role_requirement: RoleRequirement = RoleRequirement.ALL,
):
    """
    Decorator to authorize a route
    :param scopes: list of scopes
    :param roles: list of roles to validate
    :param role_requirement: role requirement (RoleRequirement.ALL or RoleRequirement.ANY)
    """

    def wrapper(endpoint):
        @wraps(endpoint)
        @requires(scopes)
        async def require_auth_endpoint(request: Request, *args, **kwargs):
            # Check args
            mandatory_roles_list = []
            if roles is not None:
                mandatory_roles_list = [roles] if isinstance(roles, str) else list(roles)

            # Since we used @requires and the caller has
            # the Depends(aad.oauth2), the request.user exists.
            user: AadUser = request.user
            user_roles_list = user.roles_id or []

            role_validator = RoleValidator(mandatory_roles_list, role_requirement)
            if len(mandatory_roles_list) > 0 and not role_validator.validate_roles(
                user_roles_list
            ):
                raise HTTPException(status_code=403, detail="Unauthorized role")

            if inspect.iscoroutinefunction(endpoint):
                result = await endpoint(request, *args, **kwargs)
            else:
                result = endpoint(request, *args, **kwargs)

            return result

        return require_auth_endpoint

    return wrapper


def oauth2_scheme(
    options: AzureAdSettings = None, env_path: Optional[str] = None, **kwargs
):
    """get the OAuth2 schema used for API Authentication"""

    if options is None:
        options = AzureAdSettings(_env_file=env_path)

    scopes_dictionary = {}
    if options.scopes_list is not None:
        for ls in options.scopes_list:
            scopes_dictionary[ls] = ls

    return OAuth2(
        flows=OAuthFlows(
            authorizationCode=OAuthFlowAuthorizationCode(
                authorizationUrl=options.authorization_url,
                tokenUrl=options.token_url,
                scopes=scopes_dictionary,
            ),
        )
    )
